import os
import subprocess
from pathlib import Path


# Funzione per ottenere il numero totale di frame
def get_total_frame_count(video_path):
    command = [
        "ffprobe",
        "-v",
        "error",
        "-select_streams",
        "v:0",
        "-show_entries",
        "stream=r_frame_rate,duration",
        "-of",
        "default=noprint_wrappers=1:nokey=1",
        video_path,
    ]

    result = subprocess.run(command, capture_output=True, text=True)

    if result.returncode != 0:
        raise RuntimeError(
            f"Error during the execution of ffprobe: {result.stderr.strip()}"
        )
    try:
        output = result.stdout.strip().split("\n")
        frame_rate = eval(output[0])
        duration = float(output[1])
        return frame_rate, duration
    except ValueError:
        raise ValueError(
            "Unable to obtain the frame number and rate. "
            f"Output ffprobe: '{result.stdout.strip()}'"
        )


def extract_dji_metadata_stream(video_path, output_path):

    if not video_path.lower().endswith(".mp4"):
        raise ValueError("the input file must be a mp4 file.")

    if Path(output_path).is_file():
        os.remove(output_path)

    command = [
        "ffmpeg",
        "-hwaccel",
        "cuda",  # Usa accelerazione hardware
        "-loglevel",
        "error",
        "-nostdin",
        "-flush_packets",
        "1",
        "-i",
        video_path,  # Input video
        "-map",
        "0:2",
        "-c",
        "copy",
        "-f",
        "data",
        output_path,
    ]
    result = subprocess.run(command, capture_output=True, text=True)

    if result.returncode != 0:
        raise RuntimeError(
            "Error during the ffmpeg metadata stream "
            f"extraction: {result.stderr.strip()}"
        )



import xml.etree.ElementTree as ET

import gpxpy.gpx

# TODO: add tests


def write_gpx_file(output_file, gps_data, write_extensions=False):

    gpx = gpxpy.gpx.GPX()
    gpx.creator = "pyosmogps -- https://github.com/francescocaponio/pyosmogps"
    track = gpxpy.gpx.GPXTrack()
    gpx.tracks.append(track)
    segment = gpxpy.gpx.GPXTrackSegment()
    track.segments.append(segment)

    latitude = [point["latitude"] for point in gps_data]
    longitude = [point["longitude"] for point in gps_data]
    altitude = [point["altitude"] for point in gps_data]
    timeinfo = [point["timeinfo"] for point in gps_data]
    camera_acc_x = [point["camera_acc_x"] for point in gps_data]
    camera_acc_y = [point["camera_acc_y"] for point in gps_data]
    camera_acc_z = [point["camera_acc_z"] for point in gps_data]
    remote_der_x = [point["remote_der_x"] for point in gps_data]
    remote_der_y = [point["remote_der_y"] for point in gps_data]
    remote_der_z = [point["remote_der_z"] for point in gps_data]

    for i in range(len(timeinfo)):
        point = gpxpy.gpx.GPXTrackPoint(
            latitude=latitude[i],
            longitude=longitude[i],
            elevation=altitude[i],
            time=timeinfo[i],
        )

        if write_extensions:
            extensions = ET.Element("extensions")

            acc_x_ext = ET.SubElement(extensions, "acc_x")
            acc_x_ext.text = f"{camera_acc_x[i]:.3f}"

            acc_y_ext = ET.SubElement(extensions, "acc_y")
            acc_y_ext.text = f"{camera_acc_y[i]:.3f}"

            acc_z_ext = ET.SubElement(extensions, "acc_z")
            acc_z_ext.text = f"{camera_acc_z[i]:.3f}"

            der_x_ext = ET.SubElement(extensions, "der_x")
            der_x_ext.text = f"{remote_der_x[i]:.3f}"

            der_y_ext = ET.SubElement(extensions, "der_y")
            der_y_ext.text = f"{remote_der_y[i]:.3f}"

            der_z_ext = ET.SubElement(extensions, "der_z")
            der_z_ext.text = f"{remote_der_z[i]:.3f}"

            point.extensions.append(extensions)
        segment.points.append(point)

    with open(output_file, "w") as gpx_file:
        gpx_file.write(gpx.to_xml())



import argparse
import logging
import logging.config
import os
import sys
import tempfile
from pathlib import Path

import pyosmogps

from .data_filters import resample_gps_data
from .ffmpeg_manager import extract_dji_metadata_stream, get_total_frame_count
from .gpx_manager import write_gpx_file
from .metadata_manager import extract_gps_info

logger = logging.getLogger(__name__)  # pylint: disable=C0103


def _make_parser() -> argparse.ArgumentParser:
    # Separated so sphinx-argparse-cli can do its auto documentation magic.
    parser = argparse.ArgumentParser(
        description="Extract the GPS data from the Osmo Action video files",
        prog="pyosmogps",
    )
    parser.add_argument(
        "command",
        choices=["extract", "merge"],
        help="Specify the command to run: 'extract' to extract "
        "GPS data or 'merge' to merge GPX files.",
    )
    parser.add_argument(
        "inputs",
        nargs="+",
        help="Input file(s). Accepts a single file or multiple files.",
    )
    parser.add_argument(
        "output",
        help="Output file. Accepts a single file or multiple files.",
    )
    parser.add_argument(
        "--frequency",
        "-f",
        type=float,
        default=2.0,
        help="Set the output data frequency in Hz (default: 2 Hz).",
    )
    parser.add_argument(
        "--resampling-method",
        "-r",
        choices=["discard", "linear", "lpf"],
        default="interpolate",
        help="Set the method for resampling data: 'discard' to drop "
        "excess samples, 'linear' for linear interpolation, 'lpf' "
        "for low pass filtering (default: interpolate).",
    )
    parser.add_argument(
        "--timezone-offset",
        "-t",
        type=int,
        default=0,
        help="Set the timezone offset in hours (default: 0).",
    )
    parser.add_argument(
        "--version", "-v", action="version", version=f"%(prog)s {pyosmogps.__version__}"
    )
    return parser


def extract(
    inputs, output_file, output_frequency, resampling_method, timezone_offset=0
):
    print(f"Running extract command with inputs: {inputs} and output: {output_file}")

    global_gps_info = []
    for i, input_file in enumerate(inputs, start=1):
        print(f"Processing file {i}/{len(inputs)}: {input_file} -> {output_file}")

        input_frame_rate, video_duration = get_total_frame_count(input_file)
        print(f"Frame rate: {input_frame_rate}, duration: {video_duration}")

        temp_file = os.path.join(tempfile.gettempdir(), Path(input_file).stem + ".tmp")
        extract_dji_metadata_stream(input_file, temp_file)

        gps_info = extract_gps_info(temp_file, timezone_offset)
        print(f"Extracted {len(gps_info)} GPS data points.")

        try:
            os.remove(temp_file)
        except FileNotFoundError:
            pass

        global_gps_info.extend(gps_info)

    global_gps_info = resample_gps_data(
        global_gps_info, input_frame_rate, output_frequency, resampling_method
    )

    if global_gps_info != []:
        write_gpx_file(output_file, global_gps_info)
        print(f"GPS data written to {output_file}")
        return True
    else:
        print("No GPS data extracted.")
        return False


def main() -> int:
    parser = _make_parser()
    if len(sys.argv) < 2:
        parser.print_help()
        parser.exit()

    args = parser.parse_args()

    if args.command == "extract":
        if not args.inputs or not args.output:
            parser.error(
                "'extract' command requires at least one input file and one "
                "output file."
            )
        success = extract(
            args.inputs, args.output, args.frequency, args.resampling_method
        )
        return 0 if success else 1

    elif args.command == "merge":
        print("Running merge command...")
        # TODO: Implement merge command
    else:
        parser.print_help()
        parser.exit()

    return 0


if __name__ == "__main__":
    exit(main())



import logging
from datetime import timedelta

from dateutil import parser

from .dji_pb2 import GenericMessage

logger = logging.getLogger(__name__)  # pylint: disable=C0103

# TODO: add tests

# Supported models:
# dvtm_ac203.proto is the Osmo Action 4 camera model
# dvtm_ac204.proto is the Osmo Action 5 camera model (untested, as I don't have one)
supported_models = ["dvtm_ac203.proto", "dvtm_ac204.proto"]


def check_camera_model(message):
    """Check the camera model from the message."""
    # camera name is a string that contains the camera model
    # we do the compatibility check on the proto_name,
    # like is done in the exiftool repository:
    # https://exiftool.org/TagNames/DJI.html#Protobuf

    try:
        camera_model = message.video_global_info.module_info[0].camera_name
    except Exception as e:
        logger.error(f"Error during the camera model extraction: {e}")
        camera_model = "Unknown"
    else:
        try:
            sn = message.video_global_info.module_info[0].serial_number
            sn_string = f" (SN: {sn})"
        except Exception:
            sn_string = ""
        print(f"Detected Camera model: {camera_model}{sn_string}")
    try:
        proto_name = message.video_global_info.module_info[0].proto_name
    except Exception as e:
        logger.error(f"Error during the camera proto_name: {e}")
        proto_name = ""
    if proto_name not in supported_models:
        raise ValueError(
            "The camera model is not a supported Osmo Action camera (yet?)."
        )
    return True


def extract_gps_info(input_file, timezone_offset=0):

    with open(input_file, "rb") as f:
        data = f.read()

    message = GenericMessage()
    try:
        message.ParseFromString(data)
    except Exception as e:
        print(f"Error during the decode operation: {e}")
        exit(-1)

    check_camera_model(message)

    # TODO: check that the message contains the GPS data

    gps_data = []

    for gps in message.gps_info:
        try:
            gpsdate = parser.parse(gps.remote_gps_info.coordinates.datetime.datetime)
            homedate = gpsdate - timedelta(hours=timezone_offset)

            gps_data.append(
                {
                    "timeinfo": homedate,
                    "altitude": gps.remote_gps_info.coordinates.gps_altitude_mm / 1000,
                    "longitude": gps.remote_gps_info.coordinates.info.longitude,
                    "latitude": gps.remote_gps_info.coordinates.info.latitude,
                    "camera_acc_x": gps.camera_info.accelerometer1.x,
                    "camera_acc_y": gps.camera_info.accelerometer1.y,
                    "camera_acc_z": gps.camera_info.accelerometer1.z,
                    "camera_acc2_x": gps.camera_info.accelerometer2.x,
                    "camera_acc2_y": gps.camera_info.accelerometer2.y,
                    "camera_acc2_z": gps.camera_info.accelerometer2.z,
                    "remote_der_x": gps.remote_gps_info.derivatives.x,
                    "remote_der_y": gps.remote_gps_info.derivatives.y,
                    "remote_der_z": gps.remote_gps_info.derivatives.z,
                }
            )
        except Exception as e:
            logger.warning(f"Error parsing GPS entry: {e}")
            continue

    return gps_data
